#!/usr/bin/perl -I/local/i18n/www/teams/fr/tools/checker


use Getopt::Long;
use File::Basename;

use strict;
use diagnostics;

use PO_Reader;
use Ortho;

my $line;
my $ERRORS = 0;
my $WARNINGS = 0;

sub print_msgstr_error($$){
	my $msgstr = shift;
	my $cause = shift;

	print "\n--------------\n";
	print "Error at line: $line\n";
	print "msgstr: `$msgstr'\n";
	print "Cause: $cause\n";

	$ERRORS++;
}

sub print_msgid_msgstr_error($$$){
	my $msgid = shift;
	my $msgstr = shift;
	my $cause = shift;

	print "\n--------------\n";
	print "Error at line: $line\n";
	print "msgid: `$msgid'\nmsgstr: `$msgstr'\n";
	print "Cause: $cause\n";

	$ERRORS++;
}


sub check_tags($$){
	my ($msgid, $msgstr) = @_;

	if( ($msgid eq 'ROLES_OF_TRANSLATORS') || ($msgid eq 'CREDIT_FOR_TRANSLATORS') ){
		return 1;
	}

	#remove opening/closing tags in msgid and msgstr
	my @msgid_tags = ();
	my @msgstr_tags = ();
	while( $msgid =~ m!<(/{0,1}[a-z]+)>!g ){
		push @msgid_tags, $1;
	}
	while( $msgstr =~ m!<(/{0,1}[a-z]+)>!g ){
		push @msgstr_tags, $1;
	}
	if( scalar(@msgid_tags) != scalar(@msgstr_tags) ){
		my $id = scalar(@msgid_tags);
		my $str = scalar(@msgstr_tags);
		print_msgid_msgstr_error $msgid, $msgstr, "number of tags doesn't match ($id vs $str)";
		return 0;
		
	}else{
		while( defined( my $id = shift @msgid_tags ) ){
			my $str = shift @msgstr_tags;
			if( $id ne $str ){
				print_msgid_msgstr_error $msgid, $msgstr, "tags are not the same (`$id' vs `$str')";
			}
		}
	}
	return 1;
}


sub show_usage{
	print "$0  [--no-tags] [--no-ortho] file1 [file2 ... fileN]\n";
	exit 0;
}

my $DO_NOT_CHECK_TAGS = 0;
my $DO_NOT_CHECK_WITH_ORTHO = 0;
my $DEBUG = 0;

my $err = GetOptions(
					 "help|h",            \&show_usage,
					 "no-ortho",  \$DO_NOT_CHECK_WITH_ORTHO,
					 "no-tags",   \$DO_NOT_CHECK_TAGS,
					 "debug",  \$DEBUG,
					);

if ( $err == 0 ) {
	show_usage();
}

show_usage()
  if( scalar( @ARGV ) == 0 );

# ---------------------------------------------------------
$DO_NOT_CHECK_TAGS = 1;
$DO_NOT_CHECK_WITH_ORTHO = 0;
# ---------------------------------------------------------

PO_Reader::set_debug( $DEBUG );

Ortho::set_debug( $DEBUG );
Ortho::init( 'text' );


while( my $f = shift ){
	print "Processing $f...\n";
#	PO_Reader::po_open( $f ) or die "Can't open $f\n";
	open FILE, $f	|| die "Can't open $f";
	
#	my $msgid = PO_Reader::po_get_msgid();
#	my $msgstr = PO_Reader::po_get_msgstr();
	$ERRORS = 0;
	$WARNINGS = 0;

#	warn "No PO header ??" if( $msgstr !~ /POT/ );
	$line=0;
	while( <FILE> ){
		$line++;
#		$msgid = PO_Reader::po_get_msgid();
#		last unless defined $msgid;
#		$msgstr = PO_Reader::po_get_msgstr();
#		$line = PO_Reader::line();;

		next if ! length( $_ );
#		next if ! length( $msgstr );
		
#		if( ! $DO_NOT_CHECK_TAGS ){
#		  check_tags( $msgid, $msgstr );
#		}

		if( ! $DO_NOT_CHECK_WITH_ORTHO ){
			my $string=$_;
#			$_ = $msgstr;
			{
				if( /\G(\s*<(.*?)>)/gc ) {				# parsed an opening or closing tag 
					print STDERR "cdf1: \$1 = `$1'\n" if $DEBUG;
				}elsif( /\G(\s*<!--(.*?)>)/gc ) {			# parsed a comment
					print STDERR "cdf1b: \$1 = `$1'\n" if $DEBUG;
				}elsif( /\G(\s*([^<]*))/gc) {				# parsed some text before the next tag
					print STDERR "cdf2: \$1 = `$1'\n"  if $DEBUG;
					my $text = $1;
					next if $text =~ /^\s*$/;			# don't check empty strings
					my ($rc, @causes) = Ortho::check_text( $text, '' );
					if( ! $rc ){
						foreach my $c (@causes){
							print_msgstr_error( $text, $c );
						}
					}
				}else{
					last; # end of line has been reached
				}
				redo;
			}
#			Ortho::fix_string($string);
		}
	}
	print "----- end of file processing ($ERRORS errors, $WARNINGS warnings) ----\n\n";
#	PO_Reader::po_close();
}

